@@ -273,46 +273,46 @@ def bin_search(l, v, order=SortOrder.Ascending, search_type=SearchType.Aprox, ke
     mi = int((hi + lo) / 2)
     if order == SortOrder.Ascending:
         if key is None:
-            while hi != lo:
-                if l[mi] == v:
-                    return mi
+            while hi != lo and l[mi] != v:
                 if v > l[mi]:
                     lo = mi + 1
                 else:
                     hi = mi
                 mi = (hi + lo) // 2
+            if l[mi] == v:
+                return mi
         else:
             cv = key(l[mi])
-            while hi != lo:
-                if cv == v:
-                    return mi
+            while hi != lo and cv != v:
                 if v > cv:
                     lo = mi + 1
                 else:
                     hi = mi
                 mi = (hi + lo) // 2
                 cv = key(l[mi])
+            if cv == v:
+                return mi
     else:
         if key is None:
-            while hi != lo:
-                if l[mi] == v:
-                    return mi
+            while hi != lo and l[mi] != v:
                 if v < l[mi]:
                     lo = mi + 1
                 else:
                     hi = mi
                 mi = (hi + lo) // 2
+            if l[mi] == v:
+                return mi
         else:
             cv = key(l[mi])
-            while hi != lo:
-                if cv == v:
-                    return mi
+            while hi != lo and cv != v:
                 if v < cv:
                     lo = mi + 1
                 else:
                     hi = mi
                 mi = (hi + lo) // 2
                 cv = key(l[mi])
+            if cv == v:
+                return mi
 
     if search_type == SearchType.Aprox:
         return mi
